package de.stefandraeger.calcclient.controller;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.util.Observable;

import javax.xml.bind.JAXBContext;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.google.gson.JsonIOException;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.JsonSyntaxException;

import de.stefandraeger.calcclient.CalcClientStatic;
import de.stefandraeger.calcclient.model.CalculatorClientModel;
import de.stefandraeger.calcclient.view.CalculatorClientView;

public class CalculatorClientController extends Observable implements ActionListener {

	private static final String LINE_BREAK = "\r\n";
	private static final String EMPTY = "";

	private static final Logger logger = LogManager.getLogger(CalculatorClientController.class);

	private CalculatorClientModel model;
	private CalculatorClientView view;

	enum Action {
		PLUS("http://localhost:8080/addition"),
		MINUS("http://localhost:9090/subtraktion"),
		MULTIPLICATION(""),
		DIVISION("");

		private String baseUrl;

		Action(String url) {
			this.baseUrl = url;
		}

		public String getBaseUrl() {
			return baseUrl;
		}
	}

	private Action action;

	public CalculatorClientController(CalculatorClientModel model) {
		this.model = model;
		this.view = new CalculatorClientView(this.model);
		addListener();
	}

	private void addListener() {
		this.view.getInputTextarea();
		this.view.getOneBtn().addActionListener(this);
		this.view.getTwoBtn().addActionListener(this);
		this.view.getThreeBtn().addActionListener(this);
		this.view.getFourBtn().addActionListener(this);
		this.view.getFiveBtn().addActionListener(this);
		this.view.getSixBtn().addActionListener(this);
		this.view.getSevenBtn().addActionListener(this);
		this.view.getEightBtn().addActionListener(this);
		this.view.getNineBtn().addActionListener(this);
		this.view.getZeroBtn().addActionListener(this);

		this.view.getPlusBtn().addActionListener(this);
		this.view.getMinusBtn().addActionListener(this);
		this.view.getMultiplikationBtn().addActionListener(this);
		this.view.getDivisionBtn().addActionListener(this);
		this.view.getSummaryBtn().addActionListener(this);

		this.view.getClearBtn().addActionListener(this);
	}

	public void start() {
		this.view.setVisible(true);
		setChanged();
		notifyObservers();
	}

	public void actionPerformed(ActionEvent e) {
		String actionCommand = e.getActionCommand();
		logger.log(Level.DEBUG, "Press " + actionCommand);
		switch (actionCommand) {
		case CalcClientStatic.ACTION_NUMBER_0:
			addTextToTextArea(CalcClientStatic.NUM0);
			break;
		case CalcClientStatic.ACTION_NUMBER_1:
			addTextToTextArea(CalcClientStatic.NUM1);
			break;
		case CalcClientStatic.ACTION_NUMBER_2:
			addTextToTextArea(CalcClientStatic.NUM2);
			break;
		case CalcClientStatic.ACTION_NUMBER_3:
			addTextToTextArea(CalcClientStatic.NUM3);
			break;
		case CalcClientStatic.ACTION_NUMBER_4:
			addTextToTextArea(CalcClientStatic.NUM4);
			break;
		case CalcClientStatic.ACTION_NUMBER_5:
			addTextToTextArea(CalcClientStatic.NUM5);
			break;
		case CalcClientStatic.ACTION_NUMBER_6:
			addTextToTextArea(CalcClientStatic.NUM6);
			break;
		case CalcClientStatic.ACTION_NUMBER_7:
			addTextToTextArea(CalcClientStatic.NUM7);
			break;
		case CalcClientStatic.ACTION_NUMBER_8:
			addTextToTextArea(CalcClientStatic.NUM8);
			break;
		case CalcClientStatic.ACTION_NUMBER_9:
			addTextToTextArea(CalcClientStatic.NUM9);
			break;
		case CalcClientStatic.ACTION_NUMBER_DIV:
			addTextToTextArea(CalcClientStatic.SIGN_DIV, Boolean.TRUE);
			this.action = Action.DIVISION;
			break;
		case CalcClientStatic.ACTION_NUMBER_MINUS:
			addTextToTextArea(CalcClientStatic.SIGN_MINUS, Boolean.TRUE);
			this.action = Action.MINUS;
			break;
		case CalcClientStatic.ACTION_NUMBER_MULTI:
			addTextToTextArea(CalcClientStatic.ASTERIX, Boolean.TRUE);
			this.action = Action.MULTIPLICATION;
			break;
		case CalcClientStatic.ACTION_NUMBER_PLUS:
			addTextToTextArea(CalcClientStatic.SIGN_PLUS, Boolean.TRUE);
			this.action = Action.PLUS;
			break;
		case CalcClientStatic.ACTION_NUMBER_SUMMARY:
			addTextToTextArea(CalcClientStatic.SIGN_SUMMARY, Boolean.TRUE);
			callAction();
			break;
		case CalcClientStatic.ACTION_CLEAR:
			clearTextArea();
			break;
		}

	}

	private void clearTextArea() {
		this.view.getInputTextarea().setText(EMPTY);
		this.action = null;
		this.model.setZahl1(null);
		this.model.setZahl2(null);
	}

	private void addTextToTextArea(String value) {
		setNumber(value);
		addTextToTextArea(value, false);
	}

	private void setNumber(String number) {
		if (this.model.getZahl1() == null && this.action == null) {
			this.model.setZahl1(Integer.parseInt(number));
		} else if (this.model.getZahl1() != null && this.action == null) {
			this.model.setZahl1(Integer.parseInt(String.valueOf(this.model.getZahl1()) + number));
		} else if (this.model.getZahl2() == null && this.action != null) {
			this.model.setZahl2(Integer.parseInt(number));
		} else if (this.model.getZahl2() != null && this.action != null) {
			this.model.setZahl2(Integer.parseInt(String.valueOf(this.model.getZahl2()) + number));
		}
	}

	private void addTextToTextArea(String value, boolean lineBreak) {
		String text = this.view.getInputTextarea().getText();
		if (lineBreak) {
			text = text.concat(LINE_BREAK);
		}

		text = text.concat(value);

		if (lineBreak) {
			text = text.concat(LINE_BREAK);
		}

		this.view.getInputTextarea().setText(text);
	}

	private void callAction() {
		String uri = this.action.getBaseUrl() + "?zahl1=" + this.model.getZahl1() + "&zahl2=" + this.model.getZahl2();
		logger.log(Level.DEBUG, uri);
		try {
			URL url = new URL(uri);
			HttpURLConnection connection = (HttpURLConnection) url.openConnection();
			connection.setRequestMethod("GET");
			connection.setRequestProperty("Accept", "application/json");

			JsonParser jsonParser = new JsonParser();
			JsonObject jsonObject = (JsonObject) jsonParser
					.parse(new InputStreamReader(connection.getInputStream(), "UTF-8"));
			logger.log(Level.DEBUG, jsonObject);
			printSummary(jsonObject);
		} catch (Exception ex) {
			logger.log(Level.ERROR, ex);
		}
	}

	private void printSummary(JsonObject jsonObject) {
		addTextToTextArea(jsonObject.get("content").getAsString(), false);
	}
}
