package de.stefandraeger.calcclient.view;

import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import de.stefandraeger.calcclient.CalcClientStatic;
import de.stefandraeger.calcclient.ResourceHelper;
import de.stefandraeger.calcclient.model.CalculatorClientModel;

public class CalculatorClientView extends JFrame implements Observer {

	private static final String CLEAR = "CE";
	private static final String ABOUT = "About";

	private static final long serialVersionUID = 1L;

	private static final int TOP = 0;
	private static final int LEFT = 0;
	private static final int WIDTH = 365;
	private static final int HEIGHT = 313;

	private static final int COMPONENT_WIDTH = 82;
	private static final int COMPONENT_HEIGHT = 25;

	private JTextArea inputTextarea;
	private JScrollPane inputTextAreaScrollPane;
	private JButton clearBtn;
	private JButton zeroBtn, oneBtn, twoBtn, threeBtn, fourBtn, fiveBtn, sixBtn, sevenBtn, eightBtn, nineBtn;
	private JButton plusBtn, minusBtn, multiplikationBtn, divisionBtn;
	private JButton summaryBtn;

	private CalculatorClientModel model;

	public CalculatorClientView(CalculatorClientModel model) {
		this.model = model;
		init();
	}

	private void init() {
		this.setTitle("Taschenrechner mit Microservices");
		this.setBounds(TOP, LEFT, WIDTH, HEIGHT);
		// Fenster zentrieren
		this.setLocationRelativeTo(null);
		this.setResizable(false);
		this.setIconImage(ResourceHelper.getImageFromResource("calculator.png"));
		this.setLayout(new FlowLayout(FlowLayout.LEFT));
		initComponents();
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private void initComponents() {
		inputTextarea = new JTextArea();
		inputTextarea.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
		inputTextarea.setEditable(false);

		inputTextarea.setFont(ResourceHelper.getFontFromResource("Open24DisplaySt.ttf"));

		inputTextAreaScrollPane = new JScrollPane(inputTextarea, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		inputTextAreaScrollPane.setPreferredSize(new Dimension((COMPONENT_WIDTH * 4) + 15, COMPONENT_HEIGHT * 5));

		Dimension normalBtnDimension = new Dimension(COMPONENT_WIDTH, COMPONENT_HEIGHT);
		Dimension doppelBtnDimension = new Dimension((COMPONENT_WIDTH * 2) + 5, COMPONENT_HEIGHT);
		Dimension vierfachBtnDimension = new Dimension((COMPONENT_WIDTH * 4) + 15, COMPONENT_HEIGHT);
		clearBtn = createButton(CLEAR, CalcClientStatic.ACTION_CLEAR, vierfachBtnDimension);

		sevenBtn = createButton(CalcClientStatic.NUM7, CalcClientStatic.ACTION_NUMBER_7, normalBtnDimension);
		eightBtn = createButton(CalcClientStatic.NUM8, CalcClientStatic.ACTION_NUMBER_8, normalBtnDimension);
		nineBtn = createButton(CalcClientStatic.NUM9, CalcClientStatic.ACTION_NUMBER_9, normalBtnDimension);
		multiplikationBtn = createButton(CalcClientStatic.ASTERIX, CalcClientStatic.ACTION_NUMBER_MULTI,
				normalBtnDimension);
		fourBtn = createButton(CalcClientStatic.NUM4, CalcClientStatic.ACTION_NUMBER_4, normalBtnDimension);
		fiveBtn = createButton(CalcClientStatic.NUM5, CalcClientStatic.ACTION_NUMBER_5, normalBtnDimension);
		sixBtn = createButton(CalcClientStatic.NUM6, CalcClientStatic.ACTION_NUMBER_6, normalBtnDimension);
		divisionBtn = createButton(CalcClientStatic.SIGN_DIV, CalcClientStatic.ACTION_NUMBER_DIV, normalBtnDimension);
		oneBtn = createButton(CalcClientStatic.NUM1, CalcClientStatic.ACTION_NUMBER_1, normalBtnDimension);
		twoBtn = createButton(CalcClientStatic.NUM2, CalcClientStatic.ACTION_NUMBER_2, normalBtnDimension);
		threeBtn = createButton(CalcClientStatic.NUM3, CalcClientStatic.ACTION_NUMBER_3, normalBtnDimension);
		plusBtn = createButton(CalcClientStatic.SIGN_PLUS, CalcClientStatic.ACTION_NUMBER_PLUS, normalBtnDimension);
		zeroBtn = createButton(CalcClientStatic.NUM0, CalcClientStatic.ACTION_NUMBER_0, normalBtnDimension);
		summaryBtn = createButton(CalcClientStatic.SIGN_SUMMARY, CalcClientStatic.ACTION_NUMBER_SUMMARY,
				doppelBtnDimension);
		minusBtn = createButton(CalcClientStatic.SIGN_MINUS, CalcClientStatic.ACTION_NUMBER_MINUS, normalBtnDimension);

		this.add(inputTextAreaScrollPane);

		this.add(clearBtn);

		this.add(sevenBtn);
		this.add(eightBtn);
		this.add(nineBtn);
		this.add(multiplikationBtn);

		this.add(fourBtn);
		this.add(fiveBtn);
		this.add(sixBtn);
		this.add(divisionBtn);

		this.add(oneBtn);
		this.add(twoBtn);
		this.add(threeBtn);
		this.add(plusBtn);

		this.add(zeroBtn);
		this.add(summaryBtn);
		this.add(minusBtn);
	}

	private JButton createButton(String text, String actionCommand, Dimension preferredSize) {
		JButton btn = new JButton(text);
		btn.setPreferredSize(preferredSize);
		btn.setActionCommand(actionCommand);
		return btn;
	}

	public void update(Observable arg0, Object arg1) {
		// TODO Auto-generated method stub

	}

	public JTextArea getInputTextarea() {
		return inputTextarea;
	}

	public void setInputTextarea(JTextArea inputTextarea) {
		this.inputTextarea = inputTextarea;
	}

	public JButton getOneBtn() {
		return oneBtn;
	}

	public void setOneBtn(JButton oneBtn) {
		this.oneBtn = oneBtn;
	}

	public JButton getFourBtn() {
		return fourBtn;
	}

	public void setFourBtn(JButton fourBtn) {
		this.fourBtn = fourBtn;
	}

	public JButton getFiveBtn() {
		return fiveBtn;
	}

	public void setFiveBtn(JButton fiveBtn) {
		this.fiveBtn = fiveBtn;
	}

	public JButton getSixBtn() {
		return sixBtn;
	}

	public void setSixBtn(JButton sixBtn) {
		this.sixBtn = sixBtn;
	}

	public JButton getSevenBtn() {
		return sevenBtn;
	}

	public void setSevenBtn(JButton sevenBtn) {
		this.sevenBtn = sevenBtn;
	}

	public JButton getEightBtn() {
		return eightBtn;
	}

	public void setEightBtn(JButton eightBtn) {
		this.eightBtn = eightBtn;
	}

	public JButton getNineBtn() {
		return nineBtn;
	}

	public void setNineBtn(JButton nineBtn) {
		this.nineBtn = nineBtn;
	}

	public JButton getPlusBtn() {
		return plusBtn;
	}

	public void setPlusBtn(JButton plusBtn) {
		this.plusBtn = plusBtn;
	}

	public JButton getMinusBtn() {
		return minusBtn;
	}

	public void setMinusBtn(JButton minusBtn) {
		this.minusBtn = minusBtn;
	}

	public JButton getMultiplikationBtn() {
		return multiplikationBtn;
	}

	public void setMultiplikationBtn(JButton multiplikationBtn) {
		this.multiplikationBtn = multiplikationBtn;
	}

	public JButton getDivisionBtn() {
		return divisionBtn;
	}

	public void setDivisionBtn(JButton divisionBtn) {
		this.divisionBtn = divisionBtn;
	}

	public JButton getSummaryBtn() {
		return summaryBtn;
	}

	public void setSummaryBtn(JButton summaryBtn) {
		this.summaryBtn = summaryBtn;
	}

	public JButton getTwoBtn() {
		return twoBtn;
	}

	public void setTwoBtn(JButton twoBtn) {
		this.twoBtn = twoBtn;
	}

	public JButton getZeroBtn() {
		return zeroBtn;
	}

	public void setZeroBtn(JButton zeroBtn) {
		this.zeroBtn = zeroBtn;
	}

	public JButton getThreeBtn() {
		return threeBtn;
	}

	public void setThreeBtn(JButton threeBtn) {
		this.threeBtn = threeBtn;
	}

	public JButton getClearBtn() {
		return clearBtn;
	}

	public void setClearBtn(JButton clearBtn) {
		this.clearBtn = clearBtn;
	}

}
